local assets =
{
	Asset("ANIM", "anim/spat_basic.zip"),
	Asset("ANIM", "anim/spat_actions.zip"),
	Asset("ANIM", "anim/spat_build.zip"),
	Asset("ANIM", "anim/spat_phlegm.zip"),
	Asset("SOUND", "sound/beefalo.fsb"),
}

local prefabs =
{
	"meat",
	"poop",
	"steelwool",
	"phlegm",
	"spat_bomb",
}

SetSharedLootTable( 'babyspatloot',
{
	{'smallmeat', 1.00},
	{'smallmeat', 1.00},
	{'smallmeat', 1.00},
	{'steelwool', 0.50},
	{'phlegm',    0.50},
})
SetSharedLootTable( 'toddlerspatloot',
{
	{'meat',      1.00},
	{'smallmeat', 1.00},
	{'smallmeat', 1.00},
	{'steelwool', 1.00},
	{'phlegm',    1.00},
})
SetSharedLootTable( 'teenspatloot',
{
	{'meat',      1.00},
	{'meat',      1.00},
	{'smallmeat', 1.00},
	{'smallmeat', 1.00},
	{'steelwool', 1.00},
	{'steelwool', 1.00},
	{'phlegm',    1.00},
	{'phlegm',    1.50},
})


local sounds =
{
	walk = "dontstarve/creatures/spat/walk",
	grunt = "dontstarve/creatures/spat/grunt",
	yell = "dontstarve/creatures/spat/yell",
	hit = "dontstarve/creatures/spat/hurt",
	death = "dontstarve/creatures/spat/death",
	curious = "dontstarve/creatures/spat/curious",
	sleep = "dontstarve/creatures/spat/sleep",
	angry = "dontstarve/creatures/spat/angry",
	spit = "dontstarve/creatures/spat/spit",
	spit_hit = "dontstarve/creatures/spat/spit_hit",
}

local brain = require "brains/babyspatbrain"

local function OnAttacked(inst, data)
	inst.components.combat:SetTarget(data.attacker)
	inst.components.combat:ShareTarget(data.attacker, 30, function(dude)
		return dude:HasTag("spat") and not dude:HasTag("player") and not dude.components.health:IsDead()
	end, 5)
end

local function FollowGrownSpat(inst)
	local nearest = FindEntity(inst, 30, function(guy) return guy.components.leader and guy.components.leader:CountFollowers() < 1 end, {"spat"}, {"baby"} )
	if nearest and nearest.components.leader then
		nearest.components.leader:AddFollower(inst)
	end
end

local function Grow(inst)
	if inst.components.sleeper:IsAsleep() then
		--inst.hairGrowthPending = true
		inst.sg:GoToState("wake")
	else
		inst.sg:GoToState("attack")
		--inst.sg:GoToState("hair_growth")
	end
end

local function GetGrowTime()
	return GetRandomWithVariance(5000, 3000)
end

local function SetBaby(inst)
	local scale = .5
	inst.Transform:SetScale(scale, scale, scale)
	inst.components.lootdropper:SetChanceLootTable('babyspatloot') 
	inst.components.health:SetMaxHealth(240)
	inst.components.sleeper:SetResistance(1)
end

local function SetToddler(inst)
	local scale = 0.7
	inst.Transform:SetScale(scale, scale, scale)
	inst.components.lootdropper:SetChanceLootTable('toddlerspatloot') 
	inst.components.health:SetMaxHealth(480)
	inst.components.sleeper:SetResistance(2)
end

local function SetTeen(inst)
	local scale = 0.9
	inst.Transform:SetScale(scale, scale, scale)
	inst.components.lootdropper:SetChanceLootTable('teenspatloot') 
	inst.components.health:SetMaxHealth(720)
	inst.components.sleeper:SetResistance(2)
end

local function SetFullyGrown(inst)
	local grown = SpawnPrefab("spat")
	grown.Transform:SetPosition(inst.Transform:GetWorldPosition() )
	grown.Transform:SetRotation(inst.Transform:GetRotation() )
	inst.sg:GoToState("attack")
	--grown.sg:GoToState("hair_growth_pop")
	inst:Remove()
end

local growth_stages =
{
	{name="baby", time = GetGrowTime, fn = SetBaby},
	{name="toddler", time = GetGrowTime, fn = SetToddler, growfn = Grow},
	{name="teen", time = GetGrowTime, fn = SetTeen, growfn = Grow},
	{name="grown", time = GetGrowTime, fn = SetFullyGrown, growfn = Grow},
}

local function fn()
	local inst = CreateEntity()

	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	inst.entity:AddDynamicShadow()
	inst.entity:AddNetwork()

	inst.AnimState:SetMultColour(255/255,175/255,175/255,1)

	inst.Transform:SetSixFaced()
	inst.Transform:SetScale(0.5, 0.5, 0.5)

	inst.DynamicShadow:SetSize(2.5, 1.25)

	inst.AnimState:SetBank("spat")
	inst.AnimState:SetBuild("spat_build")
	inst.AnimState:PlayAnimation("idle_loop", true)

	inst:AddTag("spat")
	inst:AddTag("baby")
	inst:AddTag("animal")

	MakeCharacterPhysics(inst, 100, .5)

	inst.entity:SetPristine()

	if not TheWorld.ismastersim then
		return inst
	end

	inst.sounds = sounds

	inst:AddComponent("eater")
	inst.components.eater:SetDiet({ FOODTYPE.VEGGIE }, { FOODTYPE.VEGGIE })

	inst:AddComponent("combat")
	inst.components.combat.hiteffectsymbol = "spat_body"

	inst:AddComponent("health")
	inst.components.health:SetMaxHealth(240)

	inst:AddComponent("lootdropper")

	inst:AddComponent("inspectable")
	inst:AddComponent("sleeper")

	inst:AddComponent("knownlocations")

	inst:AddComponent("follower")

	inst:AddComponent("growable")
	inst.components.growable.stages = growth_stages
	--inst.components.growable.growonly = true
	inst.components.growable:SetStage(1)
	inst.components.growable:StartGrowing()

	MakeMediumBurnableCharacter(inst, "spat_body")
	MakeLargeFreezableCharacter(inst, "spat_body")

	inst:AddComponent("locomotor") -- locomotor must be constructed before the stategraph
	inst.components.locomotor.walkspeed = 3
	inst.components.locomotor.runspeed = 10

	inst:DoTaskInTime(0, FollowGrownSpat)

	inst:SetBrain(brain)

	inst:SetStateGraph("SGspat")

	inst:ListenForEvent("attacked", OnAttacked)

	return inst
end

return Prefab("babyspat", fn, assets, prefabs)
